@@ -18,6 +18,7 @@
 #pragma once
 
 #include "common.h"
+#include "context.h"
 
 #define GPU_PFN(name) __typeof__(pl_##name) *name
 struct pl_gpu_fns {
@@ -33,18 +34,19 @@ struct pl_gpu_fns {
     GPU_PFN(tex_create);
     GPU_PFN(tex_invalidate); // optional
     GPU_PFN(tex_clear);
-    GPU_PFN(tex_blit);
+    GPU_PFN(tex_blit); // optional if no blittable formats
     GPU_PFN(tex_upload);
     GPU_PFN(tex_download);
     GPU_PFN(buf_create);
     GPU_PFN(buf_write);
     GPU_PFN(buf_read);
-    GPU_PFN(buf_export); // optional if !gpu->handle_caps.shared_mem
+    GPU_PFN(buf_export); // optional if !gpu->export_caps.buf
     GPU_PFN(buf_poll); // optional: if NULL buffers are always free to use
     GPU_PFN(desc_namespace);
     GPU_PFN(pass_create);
     GPU_PFN(pass_run);
-    GPU_PFN(sync_create); // optional if !gpu->handle_caps.sync
+    GPU_PFN(sync_create); // optional if !gpu->export_caps.sync
+    GPU_PFN(tex_export); // optional if !gpu->export_caps.sync
     GPU_PFN(gpu_flush); // optional
     GPU_PFN(gpu_finish);
 };
@@ -55,10 +57,15 @@ struct pl_gpu_fns {
 void pl_gpu_destroy(const struct pl_gpu *gpu);
 
 // Returns true if the device supports interop. This is considered to be
-// the case if at least one of `gpu->handle_caps` is nonzero.
+// the case if at least one of `gpu->export/import_caps` is nonzero.
 static inline bool pl_gpu_supports_interop(const struct pl_gpu *gpu)
 {
-    return gpu->handle_caps.shared_mem || gpu->handle_caps.sync;
+    return gpu->export_caps.tex ||
+           gpu->import_caps.tex ||
+           gpu->export_caps.buf ||
+           gpu->import_caps.buf ||
+           gpu->export_caps.sync ||
+           gpu->import_caps.sync;
 }
 
 // GPU-internal helpers: these should not be used outside of GPU implementations
@@ -66,9 +73,9 @@ static inline bool pl_gpu_supports_interop(const struct pl_gpu *gpu)
 // Log some metadata about the created GPU
 void pl_gpu_print_info(const struct pl_gpu *gpu, enum pl_log_level lev);
 
-// Sort the pl_format list into an optimal order. This tries to prefer formats
-// supporting more capabilities, while also trying to maintain a sane order
-// in terms of bit depth / component index.
+// Sort the pl_fmt list into an optimal order. This tries to prefer formats
+// supporting more capabilities, while also trying to maintain a sane order in
+// terms of bit depth / component index.
 void pl_gpu_sort_formats(struct pl_gpu *gpu);
 
 // Pretty-print the format list
