@@ -121,10 +121,10 @@ void pl_vk_inst_destroy(const struct pl_vk_inst **inst_ptr)
     if (!inst)
         return;
 
-    VkDebugReportCallbackEXT debug = (VkDebugReportCallbackEXT) inst->priv;
-    if (debug) {
+    VkDebugReportCallbackEXT debug_cb = (VkDebugReportCallbackEXT) inst->priv;
+    if (debug_cb) {
         VK_LOAD_FUN(inst->instance, vkDestroyDebugReportCallbackEXT)
-        vkDestroyDebugReportCallbackEXT(inst->instance, debug, VK_ALLOC);
+        vkDestroyDebugReportCallbackEXT(inst->instance, debug_cb, VK_ALLOC);
     }
 
     vkDestroyInstance(inst->instance, VK_ALLOC);
@@ -140,6 +140,13 @@ static VkBool32 VKAPI_PTR vk_dbg_callback(VkDebugReportFlagsEXT flags,
     struct pl_context *ctx = priv;
     enum pl_log_level lev = PL_LOG_INFO;
 
+    // We will ignore errors for a designated object, but we need to explicitly
+    // handle the case where no object is designated, because errors can have no
+    // object associated with them, and we don't want to suppress those errors.
+    if (ctx->suppress_errors_for_object != VK_NULL_HANDLE &&
+        ctx->suppress_errors_for_object == obj)
+        return false;
+
     switch (flags) {
     case VK_DEBUG_REPORT_ERROR_BIT_EXT:               lev = PL_LOG_ERR;   break;
     case VK_DEBUG_REPORT_WARNING_BIT_EXT:             lev = PL_LOG_WARN;  break;
@@ -178,6 +185,10 @@ const struct pl_vk_inst *pl_vk_inst_create(struct pl_context *ctx,
     VkExtensionProperties *exts_avail = talloc_array(tmp, VkExtensionProperties, num_exts_avail);
     vkEnumerateInstanceExtensionProperties(NULL, &num_exts_avail, exts_avail);
 
+    pl_debug(ctx, "Available instance extensions:");
+    for (int i = 0; i < num_exts_avail; i++)
+        pl_debug(ctx, "    %s", exts_avail[i].extensionName);
+
     // Add mandatory extensions
     TARRAY_APPEND(tmp, exts, num_exts, VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
 
@@ -207,20 +218,46 @@ const struct pl_vk_inst *pl_vk_inst_create(struct pl_context *ctx,
         }
     }
 
-    if (params->debug) {
-        pl_info(ctx, "Enabling vulkan debug layers");
-        // Enables the LunarG standard validation layer, which
-        // is a meta-layer that loads lots of other validators
-        static const char *layers[] = {
-            "VK_LAYER_LUNARG_standard_validation",
-        };
-        info.ppEnabledLayerNames = layers;
-        info.enabledLayerCount = PL_ARRAY_SIZE(layers);
+    // Enumerate all supported layers
+    uint32_t num_layers_avail = 0;
+    vkEnumerateInstanceLayerProperties(&num_layers_avail, NULL);
+    VkLayerProperties *layers_avail = talloc_array(tmp, VkLayerProperties, num_layers_avail);
+    vkEnumerateInstanceLayerProperties(&num_layers_avail, layers_avail);
 
-        // Enable support for debug callbacks, so we get useful messages
-        TARRAY_APPEND(tmp, exts, num_exts, VK_EXT_DEBUG_REPORT_EXTENSION_NAME);
+    pl_debug(ctx, "Available layers:");
+    for (int i = 0; i < num_layers_avail; i++)
+        pl_debug(ctx, "    %s", layers_avail[i].layerName);
+
+    // Sorted by priority
+    static const char *debug_layers[] = {
+        "VK_LAYER_KHRONOS_validation",
+        "VK_LAYER_LUNARG_standard_validation",
+    };
+
+    bool debug = params->debug;
+    if (debug) {
+        for (int i = 0; i < PL_ARRAY_SIZE(debug_layers); i++) {
+            for (int n = 0; n < num_layers_avail; n++) {
+                if (strcmp(debug_layers[i], layers_avail[n].layerName) != 0)
+                    continue;
+
+                pl_info(ctx, "Enabling debug meta layer: %s", debug_layers[i]);
+                info.ppEnabledLayerNames = &debug_layers[i];
+                info.enabledLayerCount = 1;
+
+                // Enable support for debug callbacks, so we get useful messages
+                TARRAY_APPEND(tmp, exts, num_exts, VK_EXT_DEBUG_REPORT_EXTENSION_NAME);
+                goto layers_done;
+            }
+        }
+
+        // No layer found..
+        pl_warn(ctx, "API debugging requested but no debug meta layers present... ignoring");
+        debug = false;
     }
 
+layers_done:
+
     info.ppEnabledExtensionNames = exts;
     info.enabledExtensionCount = num_exts;
 
@@ -234,8 +271,8 @@ const struct pl_vk_inst *pl_vk_inst_create(struct pl_context *ctx,
         goto error;
     }
 
-    VkDebugReportCallbackEXT debug = VK_NULL_HANDLE;
-    if (params->debug) {
+    VkDebugReportCallbackEXT debug_cb = VK_NULL_HANDLE;
+    if (debug) {
         // Set up a debug callback to catch validation messages
         VkDebugReportCallbackCreateInfoEXT dinfo = {
             .sType = VK_STRUCTURE_TYPE_DEBUG_REPORT_CALLBACK_CREATE_INFO_EXT,
@@ -252,12 +289,12 @@ const struct pl_vk_inst *pl_vk_inst_create(struct pl_context *ctx,
         // can't fail because we've already successfully created an instance
         // with this extension enabled.
         VK_LOAD_FUN(inst, vkCreateDebugReportCallbackEXT)
-        vkCreateDebugReportCallbackEXT(inst, &dinfo, VK_ALLOC, &debug);
+        vkCreateDebugReportCallbackEXT(inst, &dinfo, VK_ALLOC, &debug_cb);
     }
 
     struct pl_vk_inst *pl_vk = talloc_struct(NULL, struct pl_vk_inst, {
         .instance = inst,
-        .priv = (uint64_t) debug,
+        .priv = (uint64_t) debug_cb,
         .extensions = exts,
         .num_extensions = num_exts,
     });
@@ -509,6 +546,10 @@ static bool device_init(struct vk_ctx *vk, const struct pl_vulkan_params *params
     VkExtensionProperties *exts_avail = talloc_array(tmp, VkExtensionProperties, num_exts_avail);
     VK(vkEnumerateDeviceExtensionProperties(vk->physd, NULL, &num_exts_avail, exts_avail));
 
+    PL_DEBUG(vk, "Available device extensions:");
+    for (int i = 0; i < num_exts_avail; i++)
+        PL_DEBUG(vk, "    %s", exts_avail[i].extensionName);
+
     // Add all extensions we need
     const char ***exts = &vk->exts;
     int *num_exts = &vk->num_exts;
