@@ -129,6 +129,12 @@ static bool vk_map_color_space(VkColorSpaceKHR space, struct pl_color_space *out
         return true;
 #endif
 
+#ifdef VK_AMD_display_native_hdr
+    case VK_COLOR_SPACE_DISPLAY_NATIVE_AMD:
+        // TODO
+        return false;
+#endif
+
     // Included to satisfy the switch coverage check
     case VK_COLOR_SPACE_RANGE_SIZE_KHR:
     case VK_COLOR_SPACE_MAX_ENUM_KHR:
@@ -407,6 +413,7 @@ static bool update_swapchain_info(struct priv *p, VkSwapchainCreateInfoKHR *info
     // Clamp the extent based on the supported limits
     w = PL_MIN(PL_MAX(w, caps.minImageExtent.width), caps.maxImageExtent.width);
     h = PL_MIN(PL_MAX(h, caps.minImageExtent.height), caps.maxImageExtent.height);
+    info->imageExtent = (VkExtent2D) { w, h };
 
     // We just request whatever usage we can, and let the pl_vk decide what
     // pl_tex_params that translates to. This makes the images as flexible
@@ -419,8 +426,20 @@ static bool update_swapchain_info(struct priv *p, VkSwapchainCreateInfoKHR *info
         goto error;
     }
 
+
+    // We need to intersect the swapchain supported usage flags with the
+    // format supported usage flags
     info->imageUsage = caps.supportedUsageFlags;
-    info->imageExtent = (VkExtent2D) { w, h };
+    VkFormatProperties fmtprop;
+    vkGetPhysicalDeviceFormatProperties(vk->physd, info->imageFormat, &fmtprop);
+
+#define CHECK(usage, feature) \
+    if (!((fmtprop.optimalTilingFeatures & VK_FORMAT_FEATURE_##feature##_BIT))) \
+        info->imageUsage &= ~VK_IMAGE_USAGE_##usage##_BIT
+
+    CHECK(SAMPLED, SAMPLED_IMAGE);
+    CHECK(COLOR_ATTACHMENT, COLOR_ATTACHMENT);
+    CHECK(STORAGE, STORAGE_IMAGE);
     return true;
 
 error:
@@ -447,7 +466,7 @@ static bool vk_sw_recreate(const struct pl_swapchain *sw, int w, int h)
     // than one swapchain already active, so we need to flush any pending
     // asynchronous swapchain release operations that may be ongoing
     while (p->old_swapchain)
-        vk_poll_commands(vk, 1000000); // 1 ms
+        vk_poll_commands(vk, UINT64_MAX);
 
     VkSwapchainCreateInfoKHR sinfo = p->protoInfo;
     sinfo.oldSwapchain = p->swapchain;
@@ -651,7 +670,7 @@ static void vk_sw_swap_buffers(const struct pl_swapchain *sw)
     struct priv *p = sw->priv;
 
     while (p->frames_in_flight >= p->swapchain_depth)
-        vk_poll_commands(p->vk, 1000000); // 1 ms
+        vk_poll_commands(p->vk, UINT64_MAX);
 }
 
 static bool vk_sw_resize(const struct pl_swapchain *sw, int *width, int *height)
